#include <stdlib.h>
#include <memory.h>
#include "CplexInterface.h"

#ifndef WIN32
#include <unistd.h>
#endif

void cleanup(CPXENVptr env, CPXLPptr lp)
{
#ifndef WIN32
    int status = 0;
    char errmsg[1024] = "";

    /* Free up the problem as allocated by CPXcreateprob, if necessary */
    if ( lp != NULL ) {
        status = CPXfreeprob (env, &lp);
        if ( status ) {
            printf ("CPXfreeprob failed, error code %d.\n", status);
			exit(0);
        }
    }

    /* Free up the CPLEX environment, if necessary */
    if ( env != NULL )
    {
        status = CPXcloseCPLEX (&env);

        printf ("Could not close CPLEX environment. ");
		exit(0);
    }
#endif
}

// Return CPLEX_OK if everything is OK.
// Returns ERROR_NO_AVAIABLE_LICENSE if no license avaiable
int openEnv(CPXENVptr &env)
{
//#ifndef _WIN32
	CPXCHARptr ptr;
    int status = 0;
    char errmsg[1024] = "";

    /* Initialize the CPLEX environment */
    if( env == NULL )
    {
#ifndef WIN32
		int first = 1;
		while (1)
		{
			env = CPXopenCPLEX (&status);

		   /* If an error occurs, the status value indicates the reason for
				failure.  A call to CPXgeterrorstring will produce the text of
				the error message.  Note that CPXopenCPLEX produces no output,
				so the only way to see the cause of the error is to use
				CPXgeterrorstring.  For other CPLEX routines, the errors will
				be seen if the CPX_PARAM_SCRIND indicator is set to CPX_ON.  */
			if ( env == NULL ) {
				ptr = CPXgeterrorstring(env, status, errmsg);

				//printf("Could not open CPLEX environment, status = %d. error = %s \n",status, errmsg);
				//printf("Could not open CPLEX environment.\n");
				if (status == 32201) // NO license available
				{
					if (first == 1)
						fprintf(stderr, "\nCould not open CPLEX environment, status = %d. error = %s\n",status, errmsg);
					else
						fprintf(stderr, ".");

					fflush(stderr);
					first = 0;
					sleep(10);
					//return ERROR_NO_AVAIABLE_LICENSE;
				}
				else
					exit(0);
			}
			else
			{
				break;
			}
		}
#endif
    }

	/* Create a log file */
	CPXFILEptr fp;
	fp = CPXfopen ("MyCplexLog.txt", "w");
	if (fp != NULL)
	{
		status = CPXsetlogfile (env, fp);
		if (status){
			printf ("Can't set the log file, error %d.\n", status);
		}
	}

	/* Set pricing method */
	SetPricingMethod(env, 0);

    /* Set the scaling param */
	status = CPXsetintparam (env, CPX_PARAM_SCAIND, -1);
    if ( status ) {
        printf ("Couldn't set scaling param to 1.\n");
    }

    /* Turn on data checking */
    status = CPXsetintparam (env, CPX_PARAM_DATACHECK, CPX_ON);
    if ( status ) {
        printf ("Failure to turn on data checking, error %d.\n", status);
        exit(0);
    }
	printf("** Cplex initialization completed **\n");

    /* Turn on output to the screen */

    /*status = CPXsetintparam (env, CPX_PARAM_SCRIND, CPX_ON);
    if ( status ) {
        sprintf (errmsg, "Failure to turn on screen indicator, error %d.\n", status);
        //mexErrMsgTxt(errmsg);
    }*/

//#endif

	return CPLEX_OK;
}

/* type is one of those numbers:
	0 Automatic
	1 [CPX_DPriInd_FULL] Standard dual pricing
	2 [CPX_DPriInd_STEEP] Steepest-edge pricing
	3 [CPX_DPriInd_Full_Steep] Steepest-edge pricing in slack space
	4 [CPX_DPriInd_SteepQStart] Steepest-edge pricing, unit initial norms
*/
void SetPricingMethod(CPXENVptr env, int type)
{
	int status = CPXsetintparam (env, CPX_PARAM_DPRIIND, type);
    if ( status ) {
        printf ("Couldn't set pricing method. error = %d.\n",status);
    }
}

void CreateCplexProblem(CPXENVptr env,
						CPXLPptr &lp,
						int numcols, int numrows, double *obj,
						int *beg, int *cnt, int *ind, double *val,
						double *lb, double *ub,
						double *row_lb, double *row_ub,
						char *var_type)
{
    int i,status = 0;
    char errmsg[1024] = "";

    double	*rhs = (double*)malloc(sizeof(double)*numrows);
	double *range = (double*)malloc(sizeof(double)*numrows);
    char	*sense = (char*)malloc(sizeof(char)*numrows);

	for (i=0 ; i<numrows ; i++)
	{
		if (row_lb[i] == -CPX_INFBOUND)
		{
			sense[i] = 'L';
			rhs[i] = row_ub[i];
			range[i] = 0; // not used
		}
		else if (row_ub[i] == CPX_INFBOUND)
		{
			sense[i] = 'G';
			rhs[i] = row_lb[i];
			range[i] = 0; // not used
		}
		else if (row_lb[i] == row_ub[i])
		{
			sense[i] = 'E';
			rhs[i] = row_lb[i];
			range[i] = 0; // not used
		}
		else
		{
			sense[i] = 'R';
			rhs[i] = row_lb[i];
			range[i] = (row_ub[i] - row_lb[i]);
		}
	}

    /*-------------*/
    /* Setup CPLEX */
    /*-------------*/


    /* Create the problem. */
    lp = CPXcreateprob (env, &status, "FbaProblem");

    /* A returned pointer of NULL may mean that not enough memory
        was available or there was some other problem.  In the case of
        failure, an error message will have been written to the error
        channel from inside CPLEX.  In this example, the setting of
        the parameter CPX_PARAM_SCRIND causes the error message to
        appear on stdout.  */
    if ( lp == NULL ) {
        printf ("Failed to create LP.\n");
        exit(0);
    }

    /* Now populate the problem with the data.  For building large
        problems, consider setting the row, column and nonzero growth
        parameters before performing this task. */

    status = CPXcopylp (env, lp, numcols, numrows, CPX_MAX,
                        obj, rhs, sense,
                        beg, cnt, ind, val,
                        lb, ub, range);
    if( status != 0 ) {
		CPXCHARptr ptr = CPXgeterrorstring(env, status, errmsg);

        printf("Could not populate LP. status = %d, error = %s \n",status,errmsg);
		//printf("Could not populate LP.\n");

        exit(0);
    }

	// determine if it is LP/MILP.
	int MILP=0;
	for (i=0 ; i<numcols ; i++)
	{
		if ((var_type[i]=='I')|| (var_type[i]=='B'))
			MILP=1;
		else
			if (var_type[i]!='C')
			{
				fprintf (stdout, "Variable type index %d arrived from the variable file is not C or I or B\n",i);
				exit(0);
			}
	}
	// If It is a MILP problem (MILP==1), we need to insert the variable types.
	if (MILP==1)
	{
		status = CPXcopyctype (env, lp, var_type);
		if ( status )
		{
			fprintf (stdout, "Failed to copy var_type\n");
			exit(0);
		}
	}
}


void UpdateCplexProblemBounds(CPXENVptr env, CPXLPptr lp, int num, int *idx, double *lb, double *ub)
{
	char c[5000];
	double v[5000];
	int i;

	for (i = 0 ; i < num ; i ++)
	{
		c[i] = 'L';
		v[i] = lb[i];
	}
    	int status = CPXchgbds (env, lp, num, idx, c, v);
	if( status != 0 ) {
	        printf ("Failed to update LP.\n");
        	exit(0);
	    }

	for (i = 0 ; i < num ; i ++)
	{
		c[i] = 'U';
		v[i] = ub[i];
	}
    	status = CPXchgbds (env, lp, num, idx, c, v);
	if( status != 0 ) {
	        printf ("Failed to update LP.\n");
        	exit(0);
	    }

	return;
}


void UpdateCplexProblemObj(CPXENVptr env, CPXLPptr lp, int cnt, int *indices, double *values)
{
	int status = CPXchgobj(env, lp, cnt, indices, values);
    if ( status ) {
        printf("ERROR: failed changing objective functions\n");
        exit(0);
    }
}

void UpdateCplexProblemMatrixCoef(CPXENVptr env, CPXLPptr lp, int numcoefs, int *rowlist, int *collist, double *vallist)
{
    int status = CPXchgcoeflist (env, lp, numcoefs, rowlist, collist, vallist);

    if ( status ) {
        printf("ERROR: failed chaning matrix coefficients\n");
        exit(0);
    }
}

double SolveCplexProblem(CPXENVptr env, CPXLPptr lp, double *result_vector, int *solutionType)
{
    /* Optimize the problem and obtain solution. */
    int status = CPXlpopt (env, lp);
    char st[1024];
	CPXgetstatstring(env, status, st);
    //printf("CPXlpopt return %d (%s)\n", status, st);
    if ( status ) {
		*solutionType = -1;
		printf ("\tCPXlpopt : Failed to run optimization status = %d\n",status);
    }

	*solutionType = CPXgetstat (env, lp);
	CPXgetstatstring(env, *solutionType, st);
	printf("\tCPXgetstat return %d (%s)\n", *solutionType, st);

	double optVal;
	status = CPXsolution (env, lp, solutionType, &optVal, result_vector, NULL, NULL, NULL);
	CPXgetstatstring(env, status, st);
	//printf("CPXsolution return %d (%s)\n", status, st);
    if ( status ) {
		*solutionType = -1;
       	printf ("\tCPXsolution : Failed to obtain solution status = %d\n",status);
    }

	printf("Objective value = %e\n",  optVal);
	return optVal;
}

double SolveCplexMipProblem(CPXENVptr env, CPXLPptr lp, double *result_vector, int *solutionType, int numcols)
{
	CPXsetintparam(env,CPX_PARAM_PROBE,2);

	int time_status=CPXsetdblparam (env, CPX_PARAM_TILIM, MY_CPX_TIME_LIMIT);//time limit of 60 seconds per problem
	if (time_status)
		fprintf (stdout, "Failed to set time limit MIP - status = %d.\n",time_status);

/*	int epopt_status=CPXsetdblparam (env, CPX_PARAM_EPOPT, MY_CPX_EPOPT);
	int eprhs_status=CPXsetdblparam (env, CPX_PARAM_EPRHS, MY_CPX_EPRHS);
	printf ("changed CPX_PARAM_EPOPT to %e return value %d.\n",MY_CPX_EPOPT,epopt_status);
	printf ("changed CPX_PARAM_EPRHS to %e return value %d.\n",MY_CPX_EPRHS,eprhs_status);

	int barecomp_status=CPXsetdblparam (env, CPX_PARAM_BAREPCOMP, MY_CPX_BAREPCOMP);
	int epagap_status=CPXsetdblparam (env, CPX_PARAM_EPAGAP, MY_CPX_EPAGAP);
	int epgap_status=CPXsetdblparam (env, CPX_PARAM_EPGAP, MY_CPX_EPGAP);
	printf ("changed CPX_PARAM_BAREPCOMP to %e return value %d.\n",MY_CPX_BAREPCOMP,barecomp_status);
	printf ("changed CPX_PARAM_EPAGAP to %e return value %d.\n",MY_CPX_EPAGAP,epagap_status);
	printf ("changed CPX_PARAM_EPGAP to %e return value %d.\n",MY_CPX_EPGAP,epgap_status);

	int epmrk_status=CPXsetdblparam (env, CPX_PARAM_EPMRK, MY_CPX_EPMRK);
	int epper_status=CPXsetdblparam (env, CPX_PARAM_EPPER, MY_CPX_EPPER);
	printf ("changed CPX_PARAM_EPMRK to %e return value %d.\n",MY_CPX_EPMRK,epmrk_status);
	printf ("changed CPX_PARAM_EPPER to %e return value %d.\n",MY_CPX_EPPER,epper_status);
*/
	//int barmaxcor_status=CPXsetdblparam (env, CPX_PARAM_BARMAXCOR, MY_CPX_BARMAXCOR);

	// TOMER
	//int epint_status=CPXsetdblparam (env, CPX_PARAM_EPINT, MY_CPX_EPINT);

	//printf ("changed CPX_PARAM_BARMAXCOR to %d return value %d.\n",MY_CPX_BARMAXCOR,barmaxcor_status);

	// TOMER
	//printf ("changed CPX_PARAM_EPINT to %e return value %d.\n",MY_CPX_EPINT,epint_status);


	/* Optimize the problem and obtain solution. */
	char st[1024];

	int status = CPXmipopt (env, lp);
	if (status != 0)
	{
		printf("\tCPXmipopt failed %d\n", status);
	}

	*solutionType = CPXgetstat (env, lp);
	CPXgetstatstring(env, *solutionType, st);
	printf("\tCPXgetstat return %d (%s)\n", *solutionType, st);

	if (*solutionType == 0)
		return -1;

	double optVal;
	status = CPXgetmipobjval (env, lp, &optVal);
	if ( status )
	{
		fprintf (stdout,"\t CPXgetmipobjval failed: status = %d\n",status);
		optVal = 0;
		*solutionType = 0;
		return -1;
	}

	if (result_vector != NULL)
	{
		status = CPXgetmipx (env, lp, result_vector, 0, numcols-1);
//		for (int i = 0 ; i < 15 ; i ++)
//			printf("CPXgetmipx: %d = %f\n", i, result_vector[i]);
		if (status != 0)
		{
			fprintf (stdout, "\tCPXgetmipx failed: status = %d\n",status);
		}
	}

	printf("\tObjective value = %e\n",  optVal);

	return optVal;
}
