function [model_list  batch_run] = CBM_Exp_Run(model_list, genes, reactions_activity, reactions_direction, batch_max_size, ...
    batch_machine_num, batch_name,m_pwd)
%Description: run simulation on a list of "expression integrated human
%models". For each model in the list calculate the activity score of the
%genes and reactions that are provided as input (rxn and gene indices should be provided).
%Input:
    %model_list
    %genes - a binary list of the length of model.genes. 1 for genes for which an activity score should be
    %calculated, 0 otherwise.
    %reactions_activity- a binary list of the length of model.rxns. 1 for reactions for which an activity score should be
    %calculated, 0 otherwise
    %reactions_direction-a binary list of the length of model.rxns. 1 for reactions for which an activity score should be
    %calculated for both directions. 0 otherwise.
    %batch_max_size- 
    %batch_machine_num- number of computers/jobs/cplex license to be used
    %batch_name
    %m_pwd  path name
  %Output:
    %model_list- extended version of model_list, where for each model batch
    %run data was added 
    %batch_run

    fprintf('** CBM_EXP_RUN **\n');
    for i=1:length(model_list)
        model_list{i} = ComputeActivityScores(model_list{i}, genes, reactions_activity, reactions_direction,m_pwd);
        fprintf('Model %d: optimization num = %d\n', i, length(model_list{i}.batch_ind));
    end

    batch_run = OptBatchRun(model_list, 2, batch_max_size, batch_machine_num, batch_name,m_pwd);
   
end%function

function [model] = ComputeActivityScores(model, genes, reactions_activity, reactions_direction,m_pwd)
%Description: prepare a batch run where rows lower and upper bounds are
%changed according to the activity score that should be calculated. The
%batch bound values are the delta that should be ADDED to the original
%bounds in the new problem.
    Private_DefineParameters;
    rxn_num = length(model.rxns);

    wt_result = OptBatchRun({model}, 1, 0, 0, 'opt',m_pwd);
    wt_result = wt_result{1};
    if (wt_result.result_status == -1)
        error('ERROR: failed running wt optimization');
    end
    wt_flux = wt_result.result_vector( 1:rxn_num )';
    model.wt_flux = wt_flux;
    model.wt_opt = wt_result.result_opt;
    model.wt_bin = wt_result.result_vector( rxn_num+1:end )'>0.5;
    %model.c(rxn_num+1:end) = 1 + model.wt_bin*0.001;

    if ~isempty(genes)%calc activity score for reactions in the reaction list and for those dependant on the genes in the gene list
        reactions_activity = reactions_activity | sum(model.rxnGeneMat(:, genes>0)')'>0;
    end
    
    model.reactions_forward_score = zeros(model.rxn_num, 1)+nan;
    model.reactions_backward_score = zeros(model.rxn_num, 1)+nan;
    model.reactions_inactive_score = zeros(model.rxn_num, 1)+nan;
    model.gene_ko_score = zeros(model.gene_num, 1)+nan;

    model.reactions_forward_score(model.wt_flux >= ACTIVE_FLUX) = model.wt_opt;
    model.reactions_backward_score(model.wt_flux <= -ACTIVE_FLUX) = model.wt_opt;
    model.reactions_backward_score(model.lb(1:rxn_num)>=0) = 0;
    model.reactions_inactive_score(abs(model.wt_flux) < ACTIVE_FLUX) = model.wt_opt;
    
    % FORWARD ACTIVE
    v = find( (reactions_activity & abs(wt_flux) < ACTIVE_FLUX) | ...
              (reactions_direction & wt_flux <= 0));
    n = length(v);
    batch_lb = sparse( v, [1:n]', zeros(n,1) + ACTIVE_FLUX - model.lb(v), rxn_num, n);
    batch_ub = sparse( v, [1:n]', zeros(n,1), rxn_num, n);
    batch_ind = v;
    batch_type = zeros(n,1)+BATCH_FORWARD;

    % BACKWARD ACTIVE
    v = find( model.lb(1:rxn_num)<0 & ...
              ((reactions_activity & abs(wt_flux) < ACTIVE_FLUX) | ...
              (reactions_direction & wt_flux >= 0)));
    n = length(v);
    batch_lb = [batch_lb sparse( v, [1:n]', zeros(n,1), rxn_num, n)];
    batch_ub = [batch_ub sparse( v, [1:n]', zeros(n,1) - ACTIVE_FLUX - model.ub(v), rxn_num, n)];
    batch_ind = [batch_ind; v];
    batch_type = [batch_type; zeros(n,1)+BATCH_BACKWARD];

    % INACTIVE
    v = find(reactions_activity & abs(wt_flux) >= ACTIVE_FLUX);  n = length(v);
    batch_lb = [batch_lb sparse( v, [1:n]', zeros(n,1) - model.lb(v), rxn_num, n )];
    batch_ub = [batch_ub sparse( v, [1:n]', zeros(n,1) - model.ub(v), rxn_num, n )];
    batch_ind = [batch_ind; v];
    batch_type = [batch_type; zeros(n,1)+BATCH_INACTIVE];

    % GENE KO
    if (1==0)
        for g = find(genes)'
            g_vec = ones( size(model.rxnGeneMat, 2), 1);
            g_vec(g) = 0;
            v = [];
            for r = find(model.rxnGeneMat(:, g))
                v(r,1) = 1-Private_EvalExpEquation(model.rules{r}, g_vec);  %all the effected reactions are marked by 1.      
            end
            if sum(v) > 1
                v = find(v);%effected reactions indices
                n=length(v);
                batch_lb = [batch_lb sparse( v, ones(n,1), zeros(n,1)-model.lb(v), rxn_num, 1 )];
                batch_ub = [batch_ub sparse( v, ones(n,1), zeros(n,1)-model.ub(v), rxn_num, 1 )];
                batch_ind = [batch_ind; g];
                batch_type = [batch_type; zeros(n,1) + BATCH_GENE_KO]; %PROBLEM
            else
                if sum(v) == 1
                    model.gene_ko_single_reaction(g) = find(v);%scr should be taken from inactive run of a single reeaction
                else
                    model.gene_ko_score(g) = model.wt_opt;
                end
            end
        end
    end
    model.batch_lb = batch_lb;
    model.batch_ub = batch_ub;
    model.batch_ind = batch_ind;
    model.batch_type = batch_type;
end%function